@@ -10,10 +10,10 @@ from applitools.common.utils.json_utils import JsonInclude
 from .accessibility import SessionAccessibilityStatus
 from .geometry import RectangleSize
 from .match import ImageMatchSettings
+from .ultrafastgrid import RenderBrowserInfo
 
 if TYPE_CHECKING:
     from . import ProxySettings
-    from .ultrafastgrid import RenderBrowserInfo
 
 
 __all__ = ("TestResults", "TestResultsSummary", "TestResultContainer")
@@ -235,9 +235,15 @@ class TestResults(object):
 
 @attr.s(repr=False, str=False)
 class TestResultContainer(object):
-    test_results = attr.ib()  # type: TestResults
-    browser_info = attr.ib()  # type: Optional[RenderBrowserInfo]
-    exception = attr.ib()  # type: Optional[Exception]
+    test_results = attr.ib(
+        default=None, type=TestResults, metadata={JsonInclude.THIS: True}
+    )  # type: TestResults
+    browser_info = attr.ib(
+        default=None, metadata={JsonInclude.THIS: True}
+    )  # type: Optional[RenderBrowserInfo]
+    exception = attr.ib(
+        default=None, metadata={JsonInclude.THIS: True}
+    )  # type: Optional[Exception]
 
     def __str__(self):
         browser_info = (
@@ -260,51 +266,35 @@ class TestResultContainer(object):
 
 @attr.s(repr=False, str=False)
 class TestResultsSummary(object):
-    _all_results = attr.ib()  # type: List[TestResultContainer]
-    _exceptions = attr.ib(init=False, default=0)  # type: int
-    _passed = attr.ib(init=False, default=0)  # type: int
-    _unresolved = attr.ib(init=False, default=0)  # type: int
-    _failed = attr.ib(init=False, default=0)  # type: int
-    _mismatches = attr.ib(init=False, default=0)  # type: int
-    _missing = attr.ib(init=False, default=0)  # type: int
-    _matches = attr.ib(init=False, default=0)  # type: int
+    results = attr.ib(
+        factory=list, type=TestResultContainer, metadata={JsonInclude.THIS: True}
+    )  # type: List[TestResultContainer]
+    exceptions = attr.ib(default=0, metadata={JsonInclude.THIS: True})  # type: int
+    passed = attr.ib(default=0, metadata={JsonInclude.THIS: True})  # type: int
+    unresolved = attr.ib(default=0, metadata={JsonInclude.THIS: True})  # type: int
+    failed = attr.ib(default=0, metadata={JsonInclude.THIS: True})  # type: int
+    mismatches = attr.ib(default=0, metadata={JsonInclude.THIS: True})  # type: int
+    missing = attr.ib(default=0, metadata={JsonInclude.THIS: True})  # type: int
+    matches = attr.ib(default=0, metadata={JsonInclude.THIS: True})  # type: int
 
     @property
     def all_results(self):
         # type: () -> List[TestResultContainer]
-        return self._all_results
+        return self.results
 
     def size(self):
         # type: () -> int
         return len(self)
 
-    def __attrs_post_init__(self):
-        for result_container in self._all_results:
-            if result_container and result_container.exception:
-                self._exceptions += 1
-
-            result = result_container.test_results
-            if result is None:
-                continue
-            if result.is_failed:
-                self._failed += 1
-            elif result.is_passed:
-                self._passed += 1
-            elif result.is_unresolved:
-                self._unresolved += 1
-            self._matches += result.matches
-            self._missing += result.missing
-            self._mismatches += result.mismatches
-
     def __len__(self):
-        return len(self._all_results)
+        return len(self.results)
 
     def __iter__(self):
-        return iter(self._all_results)
+        return iter(self.results)
 
     def __getitem__(self, item):
         # type: (int) -> TestResultContainer
-        return self._all_results[item]
+        return self.results[item]
 
     def __str__(self):
         return """
@@ -318,12 +308,12 @@ result summary {{
     missing={missing}
     matches={matches}
 }}""".format(
-            all_results="\n".join(map(str, self._all_results)),
-            passed=self._passed,
-            unresolved=self._unresolved,
-            failed=self._failed,
-            exceptions=self._exceptions,
-            mismatches=self._mismatches,
-            missing=self._missing,
-            matches=self._matches,
+            all_results="\n".join(map(str, self.results)),
+            passed=self.passed,
+            unresolved=self.unresolved,
+            failed=self.failed,
+            exceptions=self.exceptions,
+            mismatches=self.mismatches,
+            missing=self.missing,
+            matches=self.matches,
         )
