@@ -19,7 +19,7 @@ from .region import (
 )
 
 if TYPE_CHECKING:
-    from applitools.common.utils.custom_types import CodedRegionPadding, Num
+    from applitools.common.utils.custom_types import CodedRegionPadding, Num, Union
 
 __all__ = ("CheckSettings", "CheckSettingsValues")
 
@@ -87,6 +87,9 @@ class CheckSettingsValues(object):
     wait_before_capture = attr.ib(
         metadata={JsonInclude.NON_NONE: True}, default=None
     )  # type: Optional[int]
+    page_id = attr.ib(
+        metadata={JsonInclude.NON_NONE: True}, default=None
+    )  # type: Optional[Text]
 
 
 Self = TypeVar("Self", bound="CheckSettings")  # typedef
@@ -115,7 +118,10 @@ class CheckSettings(object):
             return self
         try:
             self.values.layout_regions = self.__regions(
-                regions, method_name="layout_regions", padding=kwargs.get("padding")
+                regions,
+                method_name="layout_regions",
+                padding=kwargs.get("padding"),
+                region_id=kwargs.get("region_id"),
             )
         except TypeError as e:
             raise_from(TypeError("Wrong argument in .layout()"), e)
@@ -134,7 +140,10 @@ class CheckSettings(object):
             return self
         try:
             self.values.strict_regions = self.__regions(
-                regions, method_name="strict_regions", padding=kwargs.get("padding")
+                regions,
+                method_name="strict_regions",
+                padding=kwargs.get("padding"),
+                region_id=kwargs.get("region_id"),
             )
         except TypeError as e:
             raise_from(TypeError("Wrong argument in .strict()"), e)
@@ -148,18 +157,24 @@ class CheckSettings(object):
             return self
         try:
             self.values.content_regions = self.__regions(
-                regions, method_name="content_regions", padding=kwargs.get("padding")
+                regions,
+                method_name="content_regions",
+                padding=kwargs.get("padding"),
+                region_id=kwargs.get("region_id"),
             )
         except TypeError as e:
             raise_from(TypeError("Wrong argument in .content()"), e)
         return self
 
     def ignore(self, *regions, **kwargs):
-        # type: (Self, *Region, **Optional[CodedRegionPadding])  -> Self
+        # type: (Self, *Region, **Union[CodedRegionPadding, Text])  -> Self
         """Adds one or more ignore regions."""
         try:
             self.values.ignore_regions = self.__regions(
-                regions, method_name="ignore_regions", padding=kwargs.get("padding")
+                regions,
+                method_name="ignore_regions",
+                padding=kwargs.get("padding"),
+                region_id=kwargs.get("region_id"),
             )
         except TypeError as e:
             raise_from(TypeError, e)
@@ -289,6 +304,11 @@ class CheckSettings(object):
         self.values.name = name
         return self
 
+    def page_id(self, page_id):
+        # type: (Self, Text) -> Self
+        self.values.page_id = page_id
+        return self
+
     def stitch_content(self, stitch_content=True):
         # type: (Self, bool)  -> Self
         self.values.stitch_content = stitch_content
@@ -303,7 +323,7 @@ class CheckSettings(object):
         self.values.wait_before_capture = milliseconds
         return self
 
-    def __regions(self, regions, method_name, padding):
+    def __regions(self, regions, method_name, padding, region_id):
         if not regions:
             raise TypeError(
                 "{name} method called without arguments!".format(name=method_name)
@@ -312,11 +332,11 @@ class CheckSettings(object):
         regions_list = getattr(self.values, method_name)
         for region in regions:
             regions_list.append(
-                self._region_provider_from(region, method_name, padding)
+                self._region_provider_from(region, method_name, padding, region_id)
             )
         return regions_list
 
-    def _region_provider_from(self, region, method_name, padding):
+    def _region_provider_from(self, region, method_name, padding, region_id):
         if isinstance(region, Region):
             return RegionByRectangle(region)
         raise TypeError(
