@@ -1,9 +1,4 @@
-from .region import (
-    FloatingRegionByElement,
-    FloatingRegionBySelector,
-    RegionByElement,
-    RegionBySelector,
-)
+from .region import FloatingRegionBySelector, RegionBySelector
 from .selenium_check_settings import FrameLocator, SeleniumCheckSettings
 from .target import Target
 
@@ -12,7 +7,5 @@ __all__ = (
     "SeleniumCheckSettings",
     "FrameLocator",
     "RegionBySelector",
-    "RegionByElement",
     "FloatingRegionBySelector",
-    "FloatingRegionByElement",
 )
