@@ -3,6 +3,7 @@ from typing import TYPE_CHECKING, Iterable, Text, Tuple, Union
 
 import attr
 import cattr
+from selenium.common.exceptions import StaleElementReferenceException
 
 from applitools.common import (
     AccessibilityGuidelinesVersion,
@@ -12,7 +13,6 @@ from applitools.common import (
 )
 from applitools.common import ChromeEmulationInfo as ApiChromeEmulationInfo
 from applitools.common import DesktopBrowserInfo, DeviceName, FloatingBounds
-from applitools.common import IosDeviceInfo
 from applitools.common import IosDeviceInfo as ApiIosDeviceInfo
 from applitools.common import IosDeviceName, IosVersion, MatchLevel, MatchResult
 from applitools.common import Region as APIRegion
@@ -27,7 +27,8 @@ from applitools.common import (
 )
 from applitools.common.utils.json_utils import attr_from_json, underscore_to_camelcase
 
-from ..common.geometry import Rectangle, RectangleSize
+from ..common.errors import DiffsFoundError, NewTestError, TestFailedError, USDKFailure
+from ..common.geometry import Rectangle
 from ..core import (
     FloatingRegionByRectangle,
     GetRegion,
@@ -42,7 +43,7 @@ from .fluent.region import AccessibilityRegionBySelector
 from .fluent.target_path import RegionLocator
 
 if TYPE_CHECKING:
-    from typing import Any, Dict, List, Optional, Union
+    from typing import Any, Dict, List, Optional
 
     from selenium.webdriver.remote.webdriver import WebDriver, WebElement
 
@@ -54,7 +55,10 @@ if TYPE_CHECKING:
     from ..common.utils.custom_types import ViewPort
     from . import OCRRegion
     from .fluent import FrameLocator, SeleniumCheckSettings
-    from .fluent.selenium_check_settings import SeleniumCheckSettingsValues
+    from .fluent.selenium_check_settings import (
+        LazyLoadOptions,
+        SeleniumCheckSettingsValues,
+    )
 
 
 @attr.s
@@ -257,6 +261,13 @@ BrowserInfo = Union[
 ]
 
 
+@attr.s
+class CodedRegionReference(object):
+    region = attr.ib()  # type: RegionReference
+    padding = attr.ib(default=None)  # type: Optional[int, dict]
+    region_id = attr.ib(default=None)  # type: Optional[Text]
+
+
 def record_convert(records):
     # type: (Optional[Iterable[VisualGridOption]]) -> Optional[Dict[Text, Any]]
     if records:
@@ -265,16 +276,15 @@ def record_convert(records):
         return None
 
 
-def optional_element_reference_convert(is_selenium, locator=None):
-    # type: (bool, Optional[RegionLocator]) -> Optional[ElementReference]
+def optional_element_reference_convert(locator=None):
+    # type: (Optional[RegionLocator]) -> Optional[ElementReference]
     if locator is None:
         return None
     else:
-        return locator.to_dict(is_selenium)
+        return locator.to_dict()
 
 
 def frame_reference_convert(
-    is_selenium,  # type: bool
     locator=None,  # type: Optional[RegionLocator]
     number=None,  # type: Optional[int]
     name=None,  # type: Optional[Text]
@@ -285,7 +295,7 @@ def frame_reference_convert(
     elif number is not None:
         return number
     else:
-        return locator.to_dict(is_selenium)
+        return locator.to_dict()
 
 
 def browsers_info_convert(browsers_info):
@@ -310,33 +320,33 @@ def browsers_info_convert(browsers_info):
             )
         else:
             raise RuntimeError("Unexpected BrowserInfo type", type(bi))
-    return result
+    return result or None
 
 
-def target_reference_convert(is_selenium, values):
-    # type: (bool, SeleniumCheckSettingsValues) -> Optional[RegionReference]
+def target_reference_convert(values):
+    # type: (SeleniumCheckSettingsValues) -> Optional[RegionReference]
     if values.target_locator:
-        return values.target_locator.to_dict(is_selenium)
+        return values.target_locator.to_dict()
     elif values.target_region:
         return Region.convert(values.target_region)
     else:
         return None
 
 
-def ocr_target_convert(is_selenium, target):
-    # type:(bool, Union[RegionLocator, WebElement, Region]) -> RegionReference
+def ocr_target_convert(target):
+    # type:(Union[RegionLocator, WebElement, Region]) -> RegionReference
     if isinstance(target, RegionLocator):
-        return target.to_dict(is_selenium)
+        return target.to_dict()
     else:
         return Region.convert(target)
 
 
-def region_references_convert(is_selenium, regions):
-    # type: (bool, List[GetRegion]) -> List[RegionReference]
+def region_references_convert(regions):
+    # type: (List[GetRegion]) -> List[RegionReference]
     results = []
     for r in regions:
         if isinstance(r, RegionBySelector):
-            results.append(r._target_path.to_dict(is_selenium))  # noqa
+            results.append(r._target_path.to_dict())  # noqa
         elif isinstance(r, RegionByRectangle):
             results.append(Region.convert(r._region))  # noqa
         else:
@@ -344,12 +354,30 @@ def region_references_convert(is_selenium, regions):
     return results
 
 
-def floating_region_references_convert(is_selenium, regions):
-    # type: (bool, List[GetRegion]) -> List[FloatingRegion]
+def padded_region_references_convert(padded_regions):
+    # type: (List[GetRegion]) -> List[CodedRegionReference]
+    results = []
+    for r in padded_regions:
+        if isinstance(r, RegionBySelector):
+            padded_region = CodedRegionReference(
+                r._target_path.to_dict(),  # noqa
+                r.padding,
+                r.region_id,
+            )
+            results.append(padded_region)
+        elif isinstance(r, RegionByRectangle):
+            results.append(CodedRegionReference(Region.convert(r._region)))  # noqa
+        else:
+            raise RuntimeError("Unexpected region type", type(r))
+    return results
+
+
+def floating_region_references_convert(regions):
+    # type: (List[GetRegion]) -> List[FloatingRegion]
     results = []
     for r in regions:
         if isinstance(r, FloatingRegionBySelector):
-            region = r._target_path.to_dict(is_selenium)  # noqa
+            region = r._target_path.to_dict()  # noqa
             bounds = r._bounds  # noqa
         elif isinstance(r, FloatingRegionByRectangle):
             region, bounds = Region.convert(r._rect), r._bounds  # noqa
@@ -359,12 +387,12 @@ def floating_region_references_convert(is_selenium, regions):
     return results
 
 
-def accessibility_region_references_convert(is_selenium, regions):
-    # type: (bool, List[GetRegion]) -> List[AccessibilityRegion]
+def accessibility_region_references_convert(regions):
+    # type: (List[GetRegion]) -> List[AccessibilityRegion]
     results = []
     for r in regions:
         if isinstance(r, AccessibilityRegionBySelector):
-            region = r._target_path.to_dict(is_selenium)  # noqa
+            region = r._target_path.to_dict()  # noqa
             type_ = r._type  # noqa
         elif isinstance(r, AccessibilityRegionByRectangle):
             region, type_ = Region.convert(r._rect), r._type  # noqa
@@ -386,10 +414,12 @@ class MatchSettings(object):
     accessibility_settings = attr.ib(
         default=None
     )  # type: Optional[AccessibilitySettings]
-    ignore_regions = attr.ib(default=None)  # type: Optional[List[RegionReference]]
-    layout_regions = attr.ib(default=None)  # type: Optional[List[RegionReference]]
-    strict_regions = attr.ib(default=None)  # type: Optional[List[RegionReference]]
-    content_regions = attr.ib(default=None)  # type: Optional[List[RegionReference]]
+    ignore_regions = attr.ib(default=None)  # type: Optional[List[CodedRegionReference]]
+    layout_regions = attr.ib(default=None)  # type: Optional[List[CodedRegionReference]]
+    strict_regions = attr.ib(default=None)  # type: Optional[List[CodedRegionReference]]
+    content_regions = attr.ib(
+        default=None
+    )  # type: Optional[List[CodedRegionReference]]
     floating_regions = attr.ib(
         default=None
     )  # type: Optional[List[Union[Region, FloatingRegion]]]
@@ -398,8 +428,8 @@ class MatchSettings(object):
     )  # type: Optional[List[Union[Region, AccessibilityRegion]]]
 
     @classmethod
-    def convert(cls, is_selenium, image_match_settings):
-        # type: (bool, Optional[ImageMatchSettings]) -> Optional[MatchSettings]
+    def convert(cls, image_match_settings):
+        # type: (Optional[ImageMatchSettings]) -> Optional[MatchSettings]
         if image_match_settings:
             return cls(
                 exact=image_match_settings.exact,
@@ -410,15 +440,23 @@ class MatchSettings(object):
                 ignore_caret=image_match_settings.ignore_caret,
                 ignore_displacements=image_match_settings.ignore_displacements,
                 accessibility_settings=image_match_settings.accessibility_settings,
-                ignore_regions=image_match_settings.ignore_regions,  # todo: verify
-                layout_regions=image_match_settings.layout_regions,
-                strict_regions=image_match_settings.strict_regions,
-                content_regions=image_match_settings.content_regions,
+                ignore_regions=padded_region_references_convert(
+                    image_match_settings.ignore_regions
+                ),
+                layout_regions=padded_region_references_convert(
+                    image_match_settings.layout_regions
+                ),
+                strict_regions=padded_region_references_convert(
+                    image_match_settings.strict_regions
+                ),
+                content_regions=padded_region_references_convert(
+                    image_match_settings.content_regions
+                ),
                 floating_regions=floating_region_references_convert(
-                    is_selenium, image_match_settings.floating_match_settings
+                    image_match_settings.floating_match_settings
                 ),
                 accessibility_regions=accessibility_region_references_convert(
-                    is_selenium, image_match_settings.accessibility
+                    image_match_settings.accessibility
                 ),
             )
         else:
@@ -525,8 +563,8 @@ class EyesConfig(
     EyesBaseConfig, EyesOpenConfig, EyesCheckConfig, EyesClassicConfig, EyesUFGConfig
 ):
     @classmethod
-    def convert(cls, is_selenium, config):
-        # type: (bool, Configuration) -> EyesConfig
+    def convert(cls, config):
+        # type: (Configuration) -> EyesConfig
         if config.cut_provider:
             cut = ImageCropRect(
                 config.cut_provider.header,
@@ -560,9 +598,7 @@ class EyesConfig(
             session_type=config.session_type,
             properties=CustomProperty.convert(config.properties),
             batch=Batch.convert(config.batch),
-            default_match_settings=MatchSettings.convert(
-                is_selenium, config.default_match_settings
-            ),
+            default_match_settings=MatchSettings.convert(config.default_match_settings),
             host_app=config.host_app,
             host_o_s=config.host_os,
             host_o_s_info=None,  # TODO: verify
@@ -609,18 +645,16 @@ class ContextReference(object):
     scroll_root_element = attr.ib(default=None)  # type: Optional[ElementReference]
 
     @classmethod
-    def convert(cls, is_selenium, frame_locators):
-        # type: (bool, List[FrameLocator]) -> List[ContextReference]
+    def convert(cls, frame_locators):
+        # type: (List[FrameLocator]) -> List[ContextReference]
         return [
             cls(
                 frame=frame_reference_convert(
-                    is_selenium,
                     frame_locator.frame_locator,
                     frame_locator.frame_index,
                     frame_locator.frame_name_or_id,
                 ),
                 scroll_root_element=optional_element_reference_convert(
-                    is_selenium,
                     frame_locator.scroll_root_locator,
                 ),
             )
@@ -649,13 +683,15 @@ class CheckSettings(MatchSettings, ScreenshotSettings):
     visual_grid_options = attr.ib(default=None)  # type: Optional[Dict[Text, Any]]
     hooks = attr.ib(default=None)  # type: Optional[CheckSettingsHooks]
     render_id = attr.ib(default=None)  # type: Optional[Text]
+    page_id = attr.ib(default=None)  # type: Optional[Text]
     variation_group_id = attr.ib(default=None)  # type: Optional[Text]
     timeout = attr.ib(default=None)  # type: Optional[int]
     wait_before_capture = attr.ib(default=None)  # type: Optional[int]
+    lazy_load = attr.ib(default=None)  # type: Optional[LazyLoadOptions]
 
     @classmethod
-    def convert(cls, is_selenium, values):
-        # type: (bool, SeleniumCheckSettingsValues) -> CheckSettings
+    def convert(cls, values):
+        # type: (SeleniumCheckSettingsValues) -> CheckSettings
         if "beforeCaptureScreenshot" in values.script_hooks:
             hooks = CheckSettingsHooks(values.script_hooks["beforeCaptureScreenshot"])
         else:
@@ -668,9 +704,11 @@ class CheckSettings(MatchSettings, ScreenshotSettings):
             visual_grid_options=record_convert(values.visual_grid_options),
             hooks=hooks,
             render_id=None,  # TODO: verify
+            page_id=values.page_id,
             variation_group_id=values.variation_group_id,
             timeout=values.timeout,
             wait_before_capture=values.wait_before_capture,
+            lazy_load=values.lazy_load,
             # MatchSettings
             exact=None,  # TODO: verify
             match_level=values.match_level,
@@ -680,29 +718,21 @@ class CheckSettings(MatchSettings, ScreenshotSettings):
             ignore_caret=values.ignore_caret,
             ignore_displacements=values.ignore_displacements,
             accessibility_settings=None,  # TODO: verify
-            ignore_regions=region_references_convert(
-                is_selenium, values.ignore_regions
-            ),
-            layout_regions=region_references_convert(
-                is_selenium, values.layout_regions
-            ),
-            strict_regions=region_references_convert(
-                is_selenium, values.strict_regions
-            ),
-            content_regions=region_references_convert(
-                is_selenium, values.content_regions
-            ),
+            ignore_regions=padded_region_references_convert(values.ignore_regions),
+            layout_regions=padded_region_references_convert(values.layout_regions),
+            strict_regions=padded_region_references_convert(values.strict_regions),
+            content_regions=padded_region_references_convert(values.content_regions),
             floating_regions=floating_region_references_convert(
-                is_selenium, values.floating_regions
+                values.floating_regions
             ),
             accessibility_regions=accessibility_region_references_convert(
-                is_selenium, values.accessibility_regions
+                values.accessibility_regions
             ),
             # ScreenshotSettings
-            region=target_reference_convert(is_selenium, values),
-            frames=ContextReference.convert(is_selenium, values.frame_chain),
+            region=target_reference_convert(values),
+            frames=ContextReference.convert(values.frame_chain),
             scroll_root_element=optional_element_reference_convert(
-                is_selenium, values.scroll_root_locator
+                values.scroll_root_locator
             ),
             fully=values.stitch_content,
         )
@@ -748,11 +778,11 @@ class OCRExtractSettings(object):
     language = attr.ib(default=None)  # type: Optional[Text]
 
     @classmethod
-    def convert(cls, is_selenium, ocr_regions):
-        # type: (bool, Tuple[OCRRegion]) -> List[OCRExtractSettings]
+    def convert(cls, ocr_regions):
+        # type: (Tuple[OCRRegion]) -> List[OCRExtractSettings]
         return [
             cls(
-                ocr_target_convert(is_selenium, region.target),
+                ocr_target_convert(region.target),
                 region._hint,
                 region._min_match,
                 region._language,
@@ -816,15 +846,15 @@ def marshal_webdriver_ref(driver):
     }
 
 
-def marshal_configuration(is_selenium, configuration):
-    # type: (bool, Configuration) -> dict
-    eyes_config = EyesConfig.convert(is_selenium, configuration)
+def marshal_configuration(configuration):
+    # type: (Configuration) -> dict
+    eyes_config = EyesConfig.convert(configuration)
     return _keys_underscore_to_camel_remove_none(cattr.unstructure(eyes_config))
 
 
-def marshal_check_settings(is_selenium, check_settings):
-    # type: (bool, SeleniumCheckSettings) -> dict
-    check_settings = CheckSettings.convert(is_selenium, check_settings.values)
+def marshal_check_settings(check_settings):
+    # type: (SeleniumCheckSettings) -> dict
+    check_settings = CheckSettings.convert(check_settings.values)
     return _keys_underscore_to_camel_remove_none(cattr.unstructure(check_settings))
 
 
@@ -840,9 +870,9 @@ def marshal_ocr_search_settings(search_settings):
     return _keys_underscore_to_camel_remove_none(cattr.unstructure(search_settings))
 
 
-def marshal_ocr_extract_settings(is_selenium, extract_settings):
-    # type: (bool, Tuple[OCRRegion]) -> dict
-    extract_settings = OCRExtractSettings.convert(is_selenium, extract_settings)
+def marshal_ocr_extract_settings(extract_settings):
+    # type: (Tuple[OCRRegion]) -> dict
+    extract_settings = OCRExtractSettings.convert(extract_settings)
     return _keys_underscore_to_camel_remove_none(cattr.unstructure(extract_settings))
 
 
@@ -923,6 +953,8 @@ def demarshal_close_manager_results(close_manager_result_dict, config):
     results = attr_from_json(dumps(close_manager_result_dict), TestResultsSummary)
     for converted_res, raw_res in zip(results, close_manager_result_dict["results"]):
         converted_res.browser_info = demarshal_browser_info(raw_res.get("browserInfo"))
+        if raw_res.get("exception"):
+            converted_res.exception = demarshal_error(raw_res["exception"])
         if converted_res.test_results:
             converted_res.test_results.set_connection_config(
                 config.server_url, config.api_key, config.proxy
@@ -935,6 +967,25 @@ def demarshal_server_info(info_dict):
     return ServerInfo(info_dict["logsDir"])
 
 
+def demarshal_error(error_dict):
+    message = error_dict["message"]
+    if message.startswith("stale element reference"):
+        return StaleElementReferenceException(message)
+    elif error_dict.get("reason") in _matching_failures:
+        return _matching_failures[error_dict["reason"]](message)
+    else:
+        # There is usually a copy of message in stack trace too, remove it
+        stack = error_dict["stack"].split(message)[-1].strip("\n")
+        return USDKFailure(message, stack)
+
+
+_matching_failures = {
+    "test different": DiffsFoundError,
+    "test failed": TestFailedError,
+    "test new": NewTestError,
+}
+
+
 def _keys_underscore_to_camel_remove_none(obj):
     if isinstance(obj, dict):
         return {
